# -*- coding: utf-8 -*
import logging
from config import BASE_DIR

# 重写FileHandler类，用于多线程中日志按天分割
import os
import time
from logging.handlers import TimedRotatingFileHandler




class SafeFileHandler (logging.FileHandler):
    def __init__(self, filename, mode, encoding=None, delay=0):
        """
        Use the specified filename for streamed logging
        """

        logging.FileHandler.__init__ (self, filename, mode='a', encoding=encoding, delay=False)

        self.mode = mode
        self.encoding = encoding
        self.suffix = "%Y-%m-%d.log"
        self.suffix_time = ""

    def emit(self, record):

        """
        Emit a record.

        Always check time
        """
        try:
            if self.check_baseFilename (record):
                self.build_baseFilename ()
            logging.FileHandler.emit (self, record)
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError (record)

    def check_baseFilename(self, record):
        """
        Determine if builder should occur.

        record is not used, as we are just comparing times,
        but it is needed so the method signatures are the same
        """
        timeTuple = time.localtime ()

        if self.suffix_time != time.strftime (self.suffix, timeTuple) or not os.path.exists (
                                self.baseFilename + '.' + self.suffix_time):
            return 1
        else:
            return 0

    def build_baseFilename(self):
        """
        do builder; in this case,
        old time stamp is removed from filename and
        a new time stamp is append to the filename
        """
        if self.stream:
            self.stream.close ()
            self.stream = None

        # remove old suffix
        if self.suffix_time != "":
            index = self.baseFilename.find ("." + self.suffix_time)
            if index == -1:
                index = self.baseFilename.rfind (".")
            self.baseFilename = self.baseFilename[:index]

        # add new suffix
        currentTimeTuple = time.localtime ()
        self.suffix_time = time.strftime (self.suffix, currentTimeTuple)
        # self.baseFilename = self.baseFilename + "." + self.suffix_time
        self.baseFilename = BASE_DIR + self.suffix_time

        self.mode = 'a'
        if not self.delay:
            self.stream = self._open ()


class Logger (object):
    # 日志级别关系映射
    level_relations = {
        'debug': logging.DEBUG,
        'info': logging.INFO,
        'warning': logging.WARNING,
        'error': logging.ERROR,
        'critical': logging.CRITICAL
    }

    def __init__(self, filename, level='info', when='D', backCount=7,
                 fmt='%(asctime)s-%(levelname)s:  %(message)s'):
        self.logger = logging.getLogger ('mylogger')
        """
            完善分割日志方法
            '%(asctime)s  -  %(pathname)s  [line:%(lineno)d]  -  %(levelname)s:  %(message)s
            """
        # 创建日志目录（路径logdir可从自定义配置文件导入，或实例化时传入）
        # self.mkdir("./logs/")

        # 设置日志格式
        format_str = logging.Formatter (fmt)
        self.logger.setLevel (self.level_relations.get (level))  # 设置日志级别
        # 指定间隔时间自动生成文件的处理器（用于debug）

        th = SafeFileHandler (filename=filename, mode='a', encoding='utf-8')

        # 设置文件里写入的格式
        th.setFormatter (format_str)
        # 把对象加到logger里
        self.logger.addHandler (th)
        th2 = logging.StreamHandler ()
        th2.setFormatter (format_str)
        th2.setLevel (self.level_relations.get (level))
        self.logger.addHandler (th2)

    def mkdir(self, path) -> None:
        # 如果不存在则创建目录
        if not os.path.exists (path):
            os.makedirs (path)
if __name__ == '__main__':

    lo=Logger(filename=BASE_DIR+'/{0}.log'.format(time.strftime('%Y-%m-%d', time.localtime(time.time()))),
                    level='info').logger
    lo.info('c')